/******************************************************************
 * GlassWindow - an open Black Glass look for widgetmaking. 	*
 *    Created by Michael Bleigh (mbleigh@cc.gatech.edu)     	*
 *	Added addSubView for 3.0 by Kurt Papke (kwpapke@gmail.com)	*
 *	Generalized image segment widths/heights				*
 *               Version 1.01 - 8/23/2005                   	*
 *			Version 3.0 - 12/22/2005				*
 *			Version 3.1 - 1/14/06
 ******************************************************************/

function GlassWindow(width, height) {
    this.images = new Array;
    this.width = width;
    this.height = height;
    this.vOffset = 0;
    this.hOffset = 0;
    this.opacity = 255;
    this.fading = false;
    this.moving = false;
    this.scaling = false;
    
    for (i = 0; i < 9; i++) {
        this.images.push(new Image());
    }
    
    // Populate with the images we want.
    this.images[0].src = "Resources/Glass/topleft.png";
    this.images[1].src = "Resources/Glass/top.png";
    this.images[2].src = "Resources/Glass/topright.png";
    this.images[3].src = "Resources/Glass/right.png";
    this.images[4].src = "Resources/Glass/bottomright.png";
    this.images[5].src = "Resources/Glass/bottom.png";
    this.images[6].src = "Resources/Glass/bottomleft.png";
    this.images[7].src = "Resources/Glass/left.png";
    this.images[8].src = "Resources/Glass/background.png";

    // calculate minimal dimensions

    this.minWidth = this.images[0].srcWidth + this.images[2].srcWidth;		// tl + tr
    this.minHeight = this.images[0].srcHeight + this.images[6].srcHeight;	// tl + bl
    
    this.updatePosition = function() {
        if (this.width < this.minWidth) {
            this.width = this.minWidth;
        }
        
        if (this.height < this.minHeight) {
            this.height = this.minHeight;
        }
        
        // Moves the top left corner.
        this.images[0].hOffset = this.hOffset;
        this.images[0].vOffset = this.vOffset;
        
        // Moves and sizes the top
        this.images[1].hOffset = this.hOffset + this.images[0].srcWidth;	// 30 --> topleft width
        this.images[1].vOffset = this.vOffset;
        this.images[1].width = this.width - this.minWidth;
        
        // Moves the top right corner.
        this.images[2].hOffset = this.hOffset + this.width - this.images[2].srcWidth;	// 30;
        this.images[2].vOffset = this.vOffset;
        
        // Moves and sizes the right border.
        this.images[3].hOffset = this.hOffset + this.width - this.images[3].srcWidth;	// 8;
        this.images[3].vOffset = this.vOffset + this.images[2].srcHeight;			// 30;
        this.images[3].height = this.height - this.minHeight;
        
        // Moves the bottom right corner.
        this.images[4].hOffset = this.hOffset + this.width - this.images[4].srcWidth;
        this.images[4].vOffset = this.vOffset + this.height - this.images[4].srcHeight;
        
        // Moves and sizes the bottom border.
        this.images[5].hOffset = this.hOffset + this.images[6].srcWidth;	// 30 --> lower left width
        this.images[5].vOffset = this.vOffset + this.height - this.images[5].srcHeight;
        this.images[5].width = this.width - this.minWidth;
        
        // Moves the bottom left corner
        this.images[6].hOffset = this.hOffset;
        this.images[6].vOffset = this.vOffset + this.height - this.images[6].srcHeight;
        
        // Moves and sizes the left border.
        this.images[7].hOffset = this.hOffset;
        this.images[7].vOffset = this.vOffset + this.images[0].srcHeight;		// 30;
        this.images[7].height = this.height - this.minHeight;
        
        // Moves and sizes the background
        this.images[8].hOffset = this.hOffset + this.images[7].srcWidth;		// 7;
        this.images[8].vOffset = this.vOffset + this.images[1].srcHeight;		// 30;
        this.images[8].width = this.width - this.images[7].srcWidth - this.images[3].srcWidth;	// 15;
        this.images[8].height = this.height - this.minHeight;
    }

    this.addSubview = function(frame) {
        for (var i = 0; i < 9; i++)
		frame.addSubview(this.images[i]);
    } // end addSubView
    
    this.removeFromSuperview = function() {
        for (var i=0; i<9; i++)
            this.images[i].removeFromSuperview();
    } // end removeFromSuperview
    
    this.setOpacity = function(newOpacity) {
        this.opacity = newOpacity;
        for (var i = 0; i < 9; i++) {
            this.images[i].opacity = "" + newOpacity;
        }
    }
    
    this.setPosition = function(x, y) {
        this.hOffset = x;
        this.vOffset = y;
        this.updatePosition();
    }
    
    this.setDimension = function(w, h) {
        this.width = w;
        this.height = h;
        this.updatePosition();
    }
    
    this.update = function() {
        if (this.images[0].opacity != this.opacity) {
            this.setOpacity(this.opacity);
        }
        
        if (this.images[0].hOffset != this.hOffset
            || this.images[0].vOffset != this.vOffset) {
            this.setPosition(this.hOffset, this.vOffset);    
        }
        
        if (this.images[1].width != this.width - this.minWidth
            || this.images[3].height != this.height - this.minHeight) {
            this.setDimension(this.width, this.height);    
        }
    }
    
    this.moveTo = function(x, y, interval) {
        this.moving = true;
        this.fromX = this.hOffset;
        this.fromY = this.vOffset;
        this.toX = x;
        this.toY = y;
        this.moveInterval = interval / 10;
        print("Moving from (" + this.fromX + ", " + this.fromY + ") to (" + this.toX + ", " + this.toY + ")");
    }
    
    this.fade = function(from, to, interval) {
        this.fading = true;
        this.fromOpacity = from;
        this.toOpacity = to;
        this.opacity = this.fromOpacity;
        this.fadeInterval = interval / 10;
        print("Fading from " + this.fromOpacity + " to " + this.toOpacity);
    }
    
    this.scaleTo = function(w, h, interval) {
        this.scaling = true;
        this.fromWidth = this.width;
        this.toWidth = w;
        this.fromHeight = this.height;
        this.toHeight = h;
        this.scaleInterval = interval / 10;
        print("Scaling from " + this.fromWidth + "x" + this.fromHeight + " to " + this.toWidth + "x" + this.toHeight);
    }
    
    this.timerUpdate = function(interval) {
        if (this.moving || this.fading || this.scaling) {
            if (this.moving) {
                if ((this.toX > this.fromX) && (this.hOffset < this.toX)
                    || (this.toX < this.fromX) && (this.hOffset > this.toX)
                    || (this.toY > this.fromY) && (this.vOffset < this.toY)
                    || (this.toY < this.fromY) && (this.vOffset > this.toY)) {
                    this.hOffset += (this.toX - this.fromX)
                        * (interval / this.moveInterval);
                    this.vOffset += (this.toY - this.fromY)
                        * (interval / this.moveInterval);
                } else {
                    this.moving = false;
                }
            }
            
            if (this.fading) {
                if ((this.toOpacity > this.fromOpacity) && (this.opacity < this.toOpacity)
                    || (this.toOpacity < this.fromOpacity) && (this.opacity > this.toOpacity)) {
                    this.opacity += (this.toOpacity - this.fromOpacity)
                        * (interval / this.fadeInterval);
                } else {
                    this.fading = false;
                }
            }
            
            if (this.scaling) {
                if ((this.toWidth > this.fromWidth) && (this.width < this.toWidth)
                    || (this.toWidth < this.fromWidth) && (this.width > this.toWidth)
                    || (this.toHeight > this.fromHeight) && (this.height < this.toHeight)
                    || (this.toHeight < this.fromHeight) && (this.height > this.toHeight)) {
                    this.width += (this.toWidth - this.fromWidth)
                        * (interval / this.scaleInterval);
                    this.height += (this.toHeight - this.fromHeight)
                        * (interval / this.scaleInterval);
                } else {
                    this.scaling = false;
                }
            }
            
            this.updatePosition();
            this.setOpacity(this.opacity);
        }
    }
    
    this.updatePosition();
}