var months = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];

function setHeaderVis(title, vis) {
	switch (title) {
		case preferences.DoneVis.title : preferences.DoneVis.value = vis; break;
		case preferences.OnholdVis.title : preferences.OnholdVis.value = vis; break;
		case preferences.TodayVis.title : preferences.TodayVis.value = vis; break;
		case preferences.ThisWeekVis.title : preferences.ThisWeekVis.value = vis; break;
		case preferences.ThisMonthVis.title : preferences.ThisMonthVis.value = vis; break;
		case preferences.ThisQuarterVis.title : preferences.ThisQuarterVis.value = vis; break;
		case preferences.ThisYearVis.title : preferences.ThisYearVis.value = vis; break;
		case preferences.SomedayVis.title : preferences.SomedayVis.value = vis; break;
		case preferences.MustVis.title : preferences.MustVis.value = vis; break;
		case preferences.ShouldVis.title : preferences.ShouldVis.value = vis; break;
		case preferences.GoodnessVis.title : preferences.GoodnessVis.value = vis; break;
		case preferences.DoubtfulVis.title : preferences.DoubtfulVis.value = vis; break;
		case preferences.StretchVis.title : preferences.StretchVis.value = vis; break;
	} // end switch
	for (i=0; i<importances.length; i++)
		importances[i].setContextMenu();
	for (i=0; i<timelines.length; i++)
		timelines[i].setContextMenu();
	redraw();
} // hide

function Header(name, tip, orient, frame, opacity) {

	this.pref = name;
	this.title = name.title;
	this.frame = frame;
	this.orient = orient;
	if (orient == "horizontal")  // horizontal axis ==> vertical band
		this.glass = new GlassWindow(colWidth, tlFrame.height);
	else // vertical
		this.glass = new GlassWindow(main_window.width, rowSpacing());
	if (arguments.length > 4) {
		this.glass.setOpacity(opacity*preferences.BkgdOpacity.value/100)
		this.opacity0 = opacity;
	}
	else {
		this.glass.setOpacity(255*preferences.BkgdOpacity.value/100);
		this.opacity0 = 255;
	}
	this.text = new Text();
	this.text.shadow = shadowDescr;
	this.text.data = this.title;
	this.text.opacity = 255;
	this.text.bgOpacity = 0;
	this.text.tooltip = tip;
	this.text.hAlign = (orient == "horizontal") ? "center" : "right";
	this.text.style = "bold";
	frame.addSubview(this.text);

	this.date = null;
	switch (this.title) {
		case preferences.TodayVis.title: this.date = new Text(); break;
		case preferences.ThisWeekVis.title:	this.date = new Text();	break;
		case preferences.ThisMonthVis.title: this.date = new Text(); break;
		case preferences.ThisQuarterVis.title: this.date = new Text(); break;
		case preferences.ThisYearVis.title: this.date = new Text();	break;
	} // end switch
	if (this.date) {
		this.date.bgOpacity = 0;
		this.date.hAlign = "center";
	}

	this.update = function() {
		this.text.color = preferences.ColorPref.value;
		this.text.size = parseInt(preferences.HeaderFontSzPref.value);
		this.text.width = -1;
		if (preferences.FontPref.value) {
			this.text.style = "bold";
			this.text.font = preferences.FontPref.value;
		}
		this.text.shadow = (preferences.DropShadow.value != "1") ? null : shadowDescr;
		if (this.date) {
			switch (this.title) {
				case preferences.TodayVis.title:
					var d = new Date();
					this.date.data = d.toLocaleDateString();
					break;
				case preferences.ThisWeekVis.title:
					const DayLength = 1000*60*60*24; // ms/day
					var d = new Date();
					var ms = d.getTime();
					var dow = d.getDay();
					ms -= dow * DayLength;
					var d0 = new Date(ms);
					var d1 = new Date(ms + 6*DayLength);
					this.date.data = months[d0.getMonth()] + " " + d0.getDate() + " - " +
						months[d1.getMonth()] + " " + d1.getDate();
					break;
				case preferences.ThisMonthVis.title:
					var d = new Date();
					this.date.data = months[d.getMonth()];
					break;
				case preferences.ThisQuarterVis.title:
					var d = new Date();
					var m = d.getMonth()/4+1;
					var y = d.getFullYear().toString();
					this.date.data = "Q" + m.toString() + "\'" + y.substring(2);
					break;
				case preferences.ThisYearVis.title:
					var d = new Date();
					this.date.data = d.getFullYear();
					break;
			} // end switch
			this.date.color = preferences.ColorPref.value;
			this.date.size = parseInt(preferences.HeaderFontSzPref.value)*0.7;
			this.date.width = -1;
			if (preferences.FontPref.value) this.date.font = preferences.FontPref.value;
		}
	} // end update

	this.setContextMenu = function() {
		var items = new Array();
		var n=0;
		items[n] = new MenuItem();
		items[n].title = "Hide " + this.title;
		items[n].enabled = true;
		items[n].checked = false;
		items[n++].onSelect = "setHeaderVis(\"" + this.title + "\", \"0\")";
		if (this.orient == "horizontal") {
			for (var c=0; c<timelines.length; c++)
				if (!timelines[c].visible()) {
					items[n] = new MenuItem();
					items[n].title = "Show " + timelines[c].title;
					items[n].enabled = true;
					items[n].checked = false;
					items[n++].onSelect = "setHeaderVis(\"" + timelines[c].title + "\", \"1\")";
				} // end if
		} // end if
		else {
			for (var r=0; r<importances.length; r++)
				if (!importances[r].visible()) {
					items[n] = new MenuItem();
					items[n].title = "Show " + importances[r].title;
					items[n].enabled = true;
					items[n].checked = false;
					items[n++].onSelect = "setHeaderVis(\"" + importances[r].title + "\", \"1\")";
				} // end if
		} // end else
		this.text.contextMenuItems = items;
	}

	this.visible = function() {
		return (this.pref.value == "1");
	} // end visible

	this.setPosition = function(h, v) {
		if (this.orient == "horizontal") {
			this.text.hOffset = h + colWidth * 0.5;
			this.text.vOffset = v;
			this.glass.setPosition(h, 0);
		}
		else {  // vertical
			this.text.hOffset = h;
			this.text.vOffset = v + (this.text.height + CellHeight)/2 - 5;
			this.glass.setPosition(0, v);
		}
		if (this.date) {
			this.date.hOffset = this.text.hOffset;
			this.date.vOffset = this.text.vOffset + this.date.size + 2;
			this.frame.addSubview(this.date);
		}
		this.glass.addSubview(this.frame);
		this.frame.addSubview(this.text);
	} // end setPosition

	this.setOpacity = function(o) {
		this.glass.setOpacity(this.opacity0*o/100);
	} // end setOpacity

	this.setHeight = function(h) {
		this.glass.height = h;
	} // end setHeight

	this.height = function() {
		return this.glass.height;
	} // end height

	this.setDimension = function(h, v) {
		this.glass.setDimension(h,v);
	} // end setDimension

	this.width = function() {
		return this.text.width;
	} // end width

	this.removeFromSuperview = function() {
		this.glass.removeFromSuperview();
		this.text.removeFromSuperview();
		if (this.date) this.date.removeFromSuperview();
	} // end removeFromSuperview

	this.update();

} // end Header
