include("glass31.js");

const IconOffset = 10;

function rowSpacing() {
	if (preferences.TaskGlass.value == "1")
		return 45;
	else if (preferences.ShowRole.value == "1")
		return parseInt(preferences.FontSzPref.value) * 2;
	else
		return parseInt(preferences.FontSzPref.value) * 1.5;
}

function Task(node) {
	this.node = node;
	this.glass = new GlassWindow(colWidth, CellHeight);	// Optimization: only create taskGlass (on lower zOrder) if visible
	this.text = new Text();
	this.text.shadow = shadowDescr;
	this.text.opacity = 255;
	this.text.bgOpacity = 0;
	this.text.hAlign = "center";
	this.text.truncation = "end";
	this.role = new Text()
	this.icon = new Image();
	this.icon.src = "Resources/OutBox.gif";
	this.icon.width = this.icon.height = 16;
	this.icon.visible = haveFocus;

	// add new attributes here like role icon

	this.setContextMenu = function(i, m) {
		if (this.text.contextMenuItems) delete this.text.contextMenuItems;
		this.text.contextMenuItems = m;
		this.text.onContextMenu = "contextMenu(" + i + ")";
		this.text.onMultiClick = "editTask(" + i + ")";
		this.icon.onMouseUp =  "if (taskList["+i+"].getAttribute(TimelineTag) != \"Done\") { taskList["+i+"].setAttribute(TimelineTag,\"Done\");contextMenu("+i+")} else deleteTask("+i+"); redraw();"
		this.text.onDragDrop = "drop(" + i + ")";
	} // end setContextMenu

	this.update = function() {
		if (this.glass.width != colWidth)
			this.glass.setDimension(colWidth, CellHeight);
		if (preferences.TaskGlass.value != "1")
			this.glass.removeFromSuperview();
		this.text.data = node.getAttribute(TitleTag);
		if (this.node.getAttribute(TimelineTag) != "Done")
			this.icon.tooltip = "Click here to move \"" + this.text.data + "\" to the Done timeline.  " +
				"To delete instead of moving to Done, right-click on the task and select Delete Task";
		else
			this.icon.tooltip = "Click here to delete \"" + this.text.data + "\"";
		this.text.color = preferences.ColorPref.value;
		this.text.size = parseInt(preferences.FontSzPref.value);
		this.text.width = colWidth - IconOffset - 6 - this.icon.width;
		if (preferences.FontPref.value){
			this.text.style = null;
			this.text.font = preferences.FontPref.value;
			}
		this.text.shadow = (preferences.DropShadow.value != "1") ? null : shadowDescr;
		this.text.tooltip = this.text.data + "\n" +
			this.node.getAttribute(TimelineTag) + "\n" +
			this.node.getAttribute(ImportanceTag) + "\n" +
			this.node.getAttribute(RoleTag) + "\n" +
			"Since " + this.node.getAttribute(EntryDateTag) + "\n";
		if (this.node.getAttribute(DoneDateTag))
			this.text.tooltip += "Completed " + this.node.getAttribute(DoneDateTag) + "\n";
		if (this.node.getAttribute(URLTag))
			this.text.tooltip += this.node.getAttribute(URLTag) + "\n";
		if (this.node.getAttribute(EffortTag))
			this.text.tooltip += this.node.getAttribute(EffortTag) + "\n";
		if (this.node.getAttribute(DifficultyTag))
			this.text.tooltip += this.node.getAttribute(DifficultyTag) + "\n";
		this.text.tooltip += "To edit this task, double-click on it";
		if (preferences.ShowRole.value == "1") {
			if (this.role == null) this.role = new Text();
			this.role.data = node.getAttribute(RoleTag);
			var ri = roleIndex(this.role.data);
			this.role.color = roleColor(ri);
			this.role.tooltip = roleTips[ri];
			this.role.size = this.text.size*0.8;
			this.role.opacity = 128;
			this.role.hAlign = "center"; // "left"
			this.role.font = this.text.font;
		} // end if
		else if (this.role) {
			this.role.removeFromSuperview();
			delete this.role;
		} // end else
	} // end update

	this.setPosition = function(h, v) {					// reference point is LL corner
		this.text.hOffset = h + colWidth/2 - this.icon.width/2;
		this.text.vOffset = v - (CellHeight - this.text.height + ((this.role)?this.role.height:0))/2 - 9;
		if (this.role) {
			this.role.hOffset = this.text.hOffset;
			this.role.vOffset = this.text.vOffset + this.role.height-2;
		}
		this.glass.setPosition(h, v-CellHeight);
		this.icon.hOffset = h + colWidth - IconOffset - this.icon.width;
		this.icon.vOffset = this.text.vOffset-this.icon.height;
	} // end setPosition

	// Optimization: check for already in/removed from sub/superview

	this.addSubview = function(frame) {
		frame.addSubview(this.text);
		if (this.role) frame.addSubview(this.role);
		if (preferences.TaskGlass.value == "1") this.glass.addSubview(frame);
		frame.addSubview(this.icon);
	} // end addSubview

	this.removeFromSuperview = function() {
		this.text.removeFromSuperview();
		if (this.role) this.role.removeFromSuperview();
		if (preferences.TaskGlass.value == "1") this.glass.removeFromSuperview();
		this.icon.removeFromSuperview();
	} // end removeFromSuperview

	this.getAttribute = function(attr) {
		return this.node.getAttribute(attr);
	} // end getAttribute

	this.setAttribute = function(attr, val) {
		if (attr == TimelineTag  &&  val == preferences.DoneVis.title)
			this.node.setAttribute(DoneDateTag, Date());
		this.node.setAttribute(attr, val);
		this.update();
	} // end setAttribute

	this.showIcons = function() {
		this.icon.visible = true;
	}

	this.hideIcons = function() {
		this.icon.visible = false;
	}

	this.browse = function() {
		openURL(this.getAttribute(URLTag));
	}

	this.update();

} // end Task